#pragma large
#pragma debug
#pragma registerbank(3)
#pragma pagelength(80) pagewidth(110)
/***********************************************************************
************************************************************************
*                                                                      *
*  B A S I C - C A N   C O N T R O L L E R                             *
*                                                                      *
************************************************************************
************************************************************************
*                                                                      *
*  Funktion:                                                           *
*                                                                      *
*     Direkte HW-Steuerung des Philips SJA100 Basic-CAN Controllers.   *
*     Ausfhrung aller Low-Level Funktionen. Dieses Modul ist das      *
*     einzige, dass direkt in die HW-Register des CAN-Controllers      *
*     schreiben darf.                                                  *
*                                                                      *
*                                                                      *
*  Filename           :  PCANCONT.C51                                  *
*                                                                      *
*  Modulename         :  PCANCONT                                      *
*                                                                      *
*  Zielsystem         :  jedes 8051 System mit einem Philips SJA100    *
*                        CAN-Controller                                *
*                                                                      *
*  Entwicklungssystem :  Keil C-51                                     *
*                                                                      *
************************************************************************
*  Versionen:                                                          *
*                                                                      *
*  0.1  25.10.1990  Th. Fischler : Erste Version                       *
*  1.0  21.01.1991  Th. Fischler : Sende-Fehler abgefangen             *
*                                  (Fehler wenn keine andere Station   *
*                                   auf Bus)                           *
*                                                                      *
*  1.1  14.02.1996  K. Birsen    : Prozedur c_clr_overrun :            *
*                                  can_reset_request auskommentiert    *
*                                  ( nach Fehler-Angaben des Herrn     *
*                                  Michael Eicher Zellweger )          *
************************************************************************
*    Copyright 1991 .. 2001 METTLER  &  FUCHS  AG,  CH-8953 Dietikon   *
************************************************************************/

/*---------------------------------------------------------------------*
 *     D E F I N I T I O N E N                                         *
 *---------------------------------------------------------------------*/

#include "abbrev.h"
#include "pcancont.h"

/* Maximale Anzahl der Sende-Versuche bei Bus-Problemen */
/* von PCACNCONF.A51                                    */
extern const unsigned char code NBR_OF_SEND_TRIALS;




/*---------------------------------------------------------------------*
 *     I M P O R T S                                                   *
 *---------------------------------------------------------------------*/


/* RTX-51 Function-Calls */
#include <rtx51.h>

#include "pcanutil.h"


/*---------------------------------------------------------------------*
 *     M O D U L G L O B A L E  V A R I A B L E N                      *
 *---------------------------------------------------------------------*/

   static byte rtxreturn;

   /* Definition der Philips SJA100 HW-Register, diese werden an die */
   /* Speicherstelle des Symbols BCAN_REG im Assembler-Modul         */
   /* BCANCONF.A51 alloziert. (absolute Variable)                    */

   extern xdata volatile reg_type can_reg;





#pragma eject
/*---------------------------------------------------------------------*
 *     L O K A L E  P R O Z E D U R E N                                *
 *---------------------------------------------------------------------*/

/*---------------------------------------------------------------------*
 *     C A N _ R E S E T _ R E Q U E S T
 *---------------------------------------------------------------------*
 *  Funktion:
 *     Setzt den CAN-Controller in den Reset-Zustand
 *
 *---------------------------------------------------------------------*
 *  Parameter:
 *     --
 *
 *---------------------------------------------------------------------*/

static void can_reset_request (void)
{
   can_reg.bcan_reg.control.bits.RR = 1;
}



#pragma eject
/*---------------------------------------------------------------------*
 *     E X P O R T I E R T E  P R O Z E D U R E N                      *
 *---------------------------------------------------------------------*/

/*---------------------------------------------------------------------*
 *     C _ F I R S T _ I N I T
 *---------------------------------------------------------------------*
 *  Funktion:
 *     Diese Routine sollte vor der ersten
 *     initialisierung des Controllers aufgerufen werden.
 *     (Sie ist vorallem aus zur Sicherstellung der Kompatibilitt zur
 *     Basic-CAN Software vorhanden).
 *
 *---------------------------------------------------------------------*
 *  Parameter:
 *     --
 *---------------------------------------------------------------------*/
void c_first_init (void)
{
   can_reset_request();
}


/*---------------------------------------------------------------------*
 *     C _ H W _ I N I T
 *---------------------------------------------------------------------*
 *  Funktion:
 *     Grundinitialisierung des SJA100 CAN-Controllers.
 *     Der CAN-Controller wird fr diese Operation angehalten.
 *     Er muss danach mit C_START wieder neu gestartet werden.
 *
 *---------------------------------------------------------------------*
 *  Parameter:
 *
 *     bus_t0_reg   : Bus-Timing Register 0 des SJA1000
 *     bus_t1_reg   : Bus-Timing Register 1 des SJA1000
 *     out_c_reg    : Output-Control Register des SJA1000
 *     clock_devider: = bit7 = 0 fr BasicCAN, =1 fr PeliCAN
 *
 *     Returnwert   : TRUE   -> OK
 *                    FALSE  -> Fehler beim Init
 *---------------------------------------------------------------------*/

byte c_hw_init (byte bus_t0_reg,
		byte bus_t1_reg,
		byte out_c_reg,
		byte clock_devider)
{
   can_reset_request();
   /* test-mode ausschalten */
   can_reg.bcan_reg.control.bits.TM = 0;
   can_reg.bcan_reg.bus_tim_0 = bus_t0_reg;
   can_reg.bcan_reg.bus_tim_1 = bus_t1_reg;
   can_reg.bcan_reg.out_contr = out_c_reg;

   can_reg.bcan_reg.clock_devider.bytes = clock_devider;

   return TRUE;
}



/*---------------------------------------------------------------------*
 *     C _ I N I T _ C O N T _ R E G
 *---------------------------------------------------------------------*
 *  Funktion:
 *     Initialisierung des CONTROL-REGISTERS des SJA1000
 *     (Festlegung der Interrupts und des Synch-Modes).
 *     Der CAN-Controller wird fr diese Operation angehalten, er muss
 *     danach mit C_START wieder neu gestartet werden.
 *
 *---------------------------------------------------------------------*
 *  Parameter:
 *
 *     cont_reg    :  Inhalt des SJA1000 Control-Registers
 *
 *---------------------------------------------------------------------*/

void c_init_cont_reg (byte cont_reg)
{
   /* Reset-Request Bit einschalten und sicherstellen dass Test-Mode */
   /* ausgeschaltet bleibt                                           */

   can_reg.bcan_reg.control.bytes = cont_reg;

   can_reg.bcan_reg.control.bits.RR = 1;
   can_reg.bcan_reg.control.bits.TM = 0;

}



/*---------------------------------------------------------------------*
 *     C _ I N I T _ A C C E P T _ R E G
 *---------------------------------------------------------------------*
 *  Funktion:
 *     Acceptance-Code und Acceptance-Mask Register initialisieren.
 *     Der CAN-Controller wird fr diese Operation angehalten, er muss
 *     danach mit C_START wieder neu gestartet werden.
 *
 *---------------------------------------------------------------------*
 *  Parameter:
 *
 *     acc_code    :  Inhalt des Acceptanc-Code Registers
 *     acc_mask    :  Inhalt des Acceptance-Mask Registers
 *
 *---------------------------------------------------------------------*/

void c_init_accept_reg (byte acc_code,
			byte acc_mask)
{
   can_reset_request();
   can_reg.bcan_reg.acc_code = acc_code;
   can_reg.bcan_reg.acc_mask = acc_mask;
}



/*---------------------------------------------------------------------*
 *     C _ S T A R T
 *---------------------------------------------------------------------*
 *  Funktion:
 *     Starten des CAN-Controllers nach Init oder nach C_STOP.
 *     Nach C_START knnen Objekte empfangen und gesendet werden.
 *
 *---------------------------------------------------------------------*
 *  Parameter:
 *
 *     --
 *
 *---------------------------------------------------------------------*/

void c_start (void)
{
   /* Das Reset request Bit kann durch einen externen Hardware Reset */
   /* ausgeschaltet bleiben, darum solange ausschalten bis der Reset */
   /* Zustand wirklich freigegeben ist                               */
   can_reset_request();
   can_reg.bcan_reg.control.bits.RR = 0;
   while (can_reg.bcan_reg.control.bits.RR) {
      can_reg.bcan_reg.control.bits.RR = 0;
      os_wait (K_TMO, 2, 0);
   }
}



/*---------------------------------------------------------------------*
 *     C _ S T O P
 *---------------------------------------------------------------------*
 *  Funktion:
 *     Temporres Anhalten des CAN-Controllers. Es kann mit C_START
 *     weitergefahren werden. Solange C_STOP aktiv ist, knnen keine
 *     Objekte empfangen oder gesendet werden.
 *
 *---------------------------------------------------------------------*
 *  Parameter:
 *
 *     --
 *
 *---------------------------------------------------------------------*/

void c_stop (void)
{
   can_reset_request();
}



/*---------------------------------------------------------------------*
 *     C _ G E T _ R E C _ I D
 *---------------------------------------------------------------------*
 *  Funktion:
 *     Abfrage des Identifiers des gerade empfangenen Objektes.
 *
 *---------------------------------------------------------------------*
 *  Parameter:
 *
 *     Returnwert     : Identifier des gerade empfangenen Objektes
 *
 *---------------------------------------------------------------------*/

unsigned long c_get_rec_id (void)
{
   unsigned long tw;

   tw =  can_reg.bcan_reg.r_ID1.ID1;
   tw = ((tw << 3) & 0x0007) + can_reg.bcan_reg.r_ID2.ID2;
   
   return tw;
}



/*---------------------------------------------------------------------*
 *     C _ R E C _ R E M O T E
 *---------------------------------------------------------------------*
 *  Funktion:
 *     Gibt zurck ob das eben empfangene Objekt ein Remote-Frame ist.
 *
 *---------------------------------------------------------------------*
 *  Parameter:
 *
 *     Returnwert   : TRUE  -> Remote-Frame
 *                    FALSE -> Data-Frame
 *
 *---------------------------------------------------------------------*/

byte c_rec_remote (void)
{
   if (can_reg.bcan_reg.r_ID2.RTR == 1 ) {
     return TRUE;
   }
   return FALSE;
}



/*---------------------------------------------------------------------*
 *     C _ G E T _ R E C _ L E N
 *---------------------------------------------------------------------*
 *  Funktion:
 *     Gibt die Datenlnge des eben empfangenen Objektes zurck.
 *
 *---------------------------------------------------------------------*
 *  Parameter:
 *
 *     Returnwert  :  Anzahl Datenbytes die empfangen wurden (0..8)
 *
 *---------------------------------------------------------------------*/

byte c_get_rec_len (void)
{
   return (can_reg.bcan_reg.r_ID2.DLC);
}



/*---------------------------------------------------------------------*
 *     C _ R E A D _ R E C _ D A T A
 *---------------------------------------------------------------------*
 *  Funktion:
 *     Lesen der Daten des eben empfangenen Objektes. Die Daten werden
 *     in den XDATA-Bereich, dessen Anfang mit dem Parameter BUF_PTR
 *     bezeichnet ist, kopiert.
 *     Es werden soviele Bytes kopiert wie tatschlich empfangen wurden
 *     (kann mit C_GET_REC_LEN gelesen werden).
 *
 *---------------------------------------------------------------------*
 *  Parameter:
 *
 *     arr       : Array im XDATA-Bereiche in den die
 *                 eben gerade empfangenen Daten kopiert werden knnen.
 *
 *---------------------------------------------------------------------*/

void c_read_rec_data (byte xdata *arr)
{
   c_fast_copy (&can_reg.bcan_reg.rec_buf[0], arr, c_get_rec_len());
}




/*---------------------------------------------------------------------*
 *     C _ R E L _ R E C _ R E G
 *---------------------------------------------------------------------*
 *  Funktion:
 *     Nach dem Empfang eines Objektes kann der Empfangsbuffer von
 *     der CPU gelesen werden. Whrend dieser Zeit kann der
 *     CAN-Controller keine neue Daten in den Buffer schreiben.
 *     Nach dem Lesen der Daten muss der Buffer wieder fr den CAN-
 *     Controller freigegeben werden.
 *
 *---------------------------------------------------------------------*
 *  Parameter:
 *
 *     --
 *
 *---------------------------------------------------------------------*/

void c_rel_rec_reg (void)
{
   can_command_type command_reg;

   command_reg.bytes = 0;
   command_reg.bits.RRB =1;
   can_reg.bcan_reg.command = command_reg;

}



/*---------------------------------------------------------------------*
 *     C _ S E N D _ O B J
 *---------------------------------------------------------------------*
 *  Funktion:
 *     Uebertragung eines Objektes zum CAN-Controller zum Senden.
 *
 *---------------------------------------------------------------------*
 *  Parameter:
 *
 *     identifier     :  Identifier des zu sendenden Objektes
 *     remote         :  TRUE  -> sende ein Remote-Frame
 *                       FALSE -> sende ein Data-Frame
 *     length         :  Anzahl Datenbytes die gesendet werden mssen
 *     arr            :  Array mit den Daten die gesendet werden sollen
 *
 *---------------------------------------------------------------------*/

byte c_send_obj (unsigned long identifier,
		 byte remote,
		 byte length,
		 byte xdata *arr)
{
   unsigned long tw;


   unsigned char nbr_of_trials;  /* Anzahl Sendeversuche bis Abbruch */
   can_command_type command_reg;

   nbr_of_trials = 0;
   /* Falls der Buffer noch besetzt ist, warten */
   while (!(can_reg.bcan_reg.status.bits.TBS)) {
	  nbr_of_trials++;
	  if (nbr_of_trials > 10) {
	 os_wait (K_TMO, 2, 0);
	 if (nbr_of_trials > NBR_OF_SEND_TRIALS+10) {
	    /* Bus wird nicht frei, vermutlich ist kein anderer Teilnehmer */
	    /* am Bus, abbrechen                                                                               */
	    return FALSE;
	 }
      }
   }

   /* Identifier in den Buffer schreiben */
   tw = identifier;
   
   can_reg.bcan_reg.t_ID1.ID1 = (tw >> 3) & 0x000000FF;   
   can_reg.bcan_reg.t_ID2.ID2 = tw & 0x00000007;

   can_reg.bcan_reg.t_ID2.RTR = 0;   
   if (remote) {
      /*  Sende remote immer mit data length field = 0 */
      can_reg.bcan_reg.t_ID2.DLC = 0;      
      /* rtr-bit setzen */
      can_reg.bcan_reg.t_ID2.RTR = 1;   
   }
   else {
      /* Data-Frame */
      can_reg.bcan_reg.t_ID2.DLC = length;      
      /* Daten in den Buffer schreiben */
      c_fast_copy (arr, &can_reg.bcan_reg.trans_buf[0], length);
   }
   /* Transmission Request Bit setzen */

   command_reg.bytes = 0;
   command_reg.bits.TR = 1;
   can_reg.bcan_reg.command = command_reg;

   return TRUE;
}



/*---------------------------------------------------------------------*
 *     C _ C L R _ O V E R R U N
 *---------------------------------------------------------------------*
 *  Funktion:
 *     Lschen des Overrun-Status im SJA1000.
 *
 *
 *
 *---------------------------------------------------------------------*
 *  Parameter:
 *
 *     --
 *
 *---------------------------------------------------------------------*/

void c_clr_overrun (void)
{
   can_command_type command_reg;

   /* Overrun-Status zurcksetzen und Receive-Buffer releasen */

   command_reg.bytes = 0;
   command_reg.bits.COS =1;
   command_reg.bits.RRB = 1;
   can_reg.bcan_reg.command = command_reg;

   /* fr die erste Controller Version kurz Controller reseten */
/*   can_reset_request();*/
   can_reg.bcan_reg.control.bits.RR = 0;
   while (can_reg.bcan_reg.control.bits.RR) {
      can_reg.bcan_reg.control.bits.RR = 0;
      os_wait (K_TMO, 2, 0);
   }
}



/*---------------------------------------------------------------------*
 *     C _ G E T _ I N T _ R E G
 *---------------------------------------------------------------------*
 *  Funktion:
 *     Diese Funktion liest den Inhalt des Interrupt-Registers
 *     des SJA1000.
 *
 *---------------------------------------------------------------------*
 *  Parameter:
 *
 *     Returnwert  :  Inhalt des SJA1000 Interrupt-Registers
 *
 *---------------------------------------------------------------------*/

byte c_get_int_reg (void)
{
   return can_reg.bcan_reg.intr.bytes;
}



/*---------------------------------------------------------------------*
 *     C _ G E T _ S T A T U S
 *---------------------------------------------------------------------*
 *  Funktion:
 *     Gibt den aktuellen Bus-Zustand des SJA1000 zurck
 *
 *---------------------------------------------------------------------*
 *  Parameter:
 *
 *     Returnwert  :  Aktueller SJA1000 Bus-Zustand
 *
 *---------------------------------------------------------------------*/

enum can_bus_stat_t c_get_status (void)
{
   if (can_reg.bcan_reg.status.bits.BS) {
      return bus_off;
   }
   if (can_reg.bcan_reg.status.bits.ES) {
      return error_passive;
   }
   return error_active;
}
