#pragma large
#pragma debug
#pragma registerbank(3)
#pragma pagelength(80) pagewidth(110)
/***********************************************************************
************************************************************************
*                                                                      *
*  C A N   O B J E C T  A D M I N I S T R A T I O N                    *
*                                                                      *
************************************************************************
************************************************************************
*                                                                      *
*  Funktion:                                                           *
*                                                                      *
*     Dieses Modul bernimmt die Verwaltung von allen vom Benutzer     *
*     definierten Kommunikationsobjekten. Es regelt den Empfang und    *
*     das Senden der Kommunikationsobjekte.                            *
*                                                                      *
*     Beim kompilieren muss ein Makro mit dem Namen                    *
*     SJA1000 angegeben werden                                         *
*                                                                      *
*                                                                      *
*  Filename           :   PCANOBJA.C51                                 *
*                                                                      *
*  Modulename         :   PCANOBJA                                     *
*                                                                      *
*  Zielsystem         :   Jedes 8051 System mit einem Philips          *
*                         SJA1000 CAN-Controller                       *
*                                                                      *
*  Entwicklungssystem :   Keil C-51                                    *
*                                                                      *
************************************************************************
*  Versionen:                                                          *
*                                                                      *
*  0.1  26.10.1990  Th. Fischler : Erste Version                       *
*  1.0  21.01.1991  Th. Fischler : Send-Error Handling eingebaut       *
*  1.1  13.02.1991  Th. Fischler : Mit Variante fr den Intel-         *
*                                  Controller erweitert                *
*  1.2  09.04.1992  Th. Fischler : Fehler in Akkzeptanzfilterung       *
*                                  und Unbind von Objekten behoben     *
*                                                                      *
************************************************************************
*    Copyright 1991 .. 2001 METTLER  &  FUCHS  AG,  CH-8953 Dietikon   *
************************************************************************/

/*---------------------------------------------------------------------*
 *     D E F I N I T I O N E N                                         *
 *---------------------------------------------------------------------*/


/* Test ob der CAN-Controller angewhlt wurde */
#ifndef SJA1000
   #error "SJA1000 muss definiert werden"
#endif



#include "abbrev.h"
#include "pcanobja.h"
#include "candefs.h"

#define MAX_OBJECTS 255


/* Maximale Lnge der Daten in einem Objekt */
#define MAX_OBJECT_DATA_SIZE 8

/* Grsster Basic CAN Objektidentifier, der benutzt werden darf */
#define MAX_OBJECT_IDENT 2031


/*---------------------------------------------------------------------*
*     I M P O R T S                                                   *
*---------------------------------------------------------------------*/

#include "pcancont.h"
#include "pcanutil.h"


/*---------------------------------------------------------------------*
*     M O D U L G L O B A L E  V A R I A B L E N                      *
*---------------------------------------------------------------------*/

/*==================================================================*/
/* NUR FUER Philips SJA1000                                          */
/*==================================================================*/
/* Die Grsse des Objektbuffers kann vom Kunden in einem eigenen    */
/* Konfigurationsfile festgelegt werden (BCANCONF.A51).             */
/* Jedes zu definierende Objekt bentigt 14 Byte XDATA-RAM          */
/* (unabhngig von der tatschlich verwendeten Datenlnge).         */
/* Der Linker alloziert diese Variable automatisch an den die       */
/* Speicherstelle, die das Symbol CAN_OBJ_BUF im Assembler Modul    */
/* BCANCONF einnimmt.                                               */
/* Hier wird die maximal mgliche Anzahl von Objekten definiert     */
/* (Korrekte Code-Generierung des Compilers !)                      */
/* Das Objekt mit dem Index 0 wird nicht verwendet !                */
/* (Dummy-Element, zum Anzeigen der Nichtexistenz eines Objektes)   */

extern xdata struct {
	     unsigned long identifier;    /* CAN Objekt-ID      */
	     byte frame;                 /* 1 fr ext, 0 fr standart */
	     byte obj_typ;               /* Typ des Objekts    */
	     byte data_length;           /* Anzahl Datenbytes  */
	     byte task_bind;             /* Task-ID oder FFH   */
	     byte obj_read;              /* Objekt gelesen ?   */
	     byte dat[MAX_OBJECT_DATA_SIZE]; /* Objekt-Daten  */
	  } can_obj_buf[MAX_OBJECTS+1];

/* Hilfsvariablen, liegen an derselben Adresse wie die Symbole    */
/* CAN_OBJ_BUF_END und CAN_OBJ_BUF_START im Assembler Modul       */
/* PCANCONF.A51                                                   */
extern xdata byte can_obj_buf_start;
extern xdata byte can_obj_buf_end;

/* Vom Benutzer definierte Lnge des Objektbuffers in Anzahl        */
/* Objekten                                                         */
static xdata unsigned int obj_buf_len;

static xdata unsigned long acc_code, acc_mask;
/* Bilden zusammen die 11-Bit Akzeptanzfilterung, die Variable      */
/* ACC_MASK gibt an, welche Bits der Variable ACC_CODE fr die      */
/* Filterung relevant sind (1->relevant, 0->nicht relevant).        */
/* Falls der Identifier eines empfangenen Objektes zumindest an den */
/* durch die Variable ACC_MASK bestimmten Stellen mit der Variablen */
/* ACC_CODE bereinstimmt, hat das Objekt die zweite Stufe der      */
/* Akzeptanzfilterung passiert (1. Stufe macht der SJA1000).         */
/*                                                                  */
/* Eine Identifiaktion (ID) passiert die Akzeptanzfilterung in der  */
/* zweiten Stufe wenn :                                             */
/*   (ID XOR ACC_CODE) AND (ACC_MASK) = 00000000000B                */

static xdata byte new_init;
/* Ist TRUE falls der Objektbuffer eben mit C_INIT_OBJ_BUFF neu     */
/* initialisiert wurde                                              */




/* Anzahl definierte Objekte                                        */
static xdata byte nbr_of_obj;

/* Enthlt die Indexe aller Objekte die an einen bestimmten Task    */
/* gebunden sind (deren Feld OBJ_BUF.TASK_BIND ungleich FFH ist).   */
/* Der Wert 0 zeigt ein unbenutztes Feld an (enthlt keinen Index). */
/* Mit Hilfe dieses Arrays kann schnell ein zu einem bestimmten     */
/* Task gehrendes Objekt gefunden werden.                          */
static xdata byte bind_index_list[MAX_BIND_OBJ];


#pragma eject
/*---------------------------------------------------------------------*
*          L O K A L E   P R O Z E D U R E N                                                       *
*---------------------------------------------------------------------*/


#pragma eject
/*---------------------------------------------------------------------*
*     E X P O R T I E R T E  P R O Z E D U R E N                      *
*---------------------------------------------------------------------*/


/*---------------------------------------------------------------------*
*     Initialisieren und Definieren von Objekten                      *
*---------------------------------------------------------------------*/

/*---------------------------------------------------------------------*
 *     C _ I N I T _ O B J _  B U F
 *---------------------------------------------------------------------*
 *  Funktion:
 *     Neu Initialisieren des Objektbuffers, Lschen aller vorhandenen
 *     Objekte
 *
 *---------------------------------------------------------------------*
 *  Parameter:
 *
 *     --
 *
 *---------------------------------------------------------------------*/

void c_init_obj_buf (void)
{
   byte i;

   obj_buf_len = ((&can_obj_buf_end+1) - &can_obj_buf_start) /
					sizeof(can_obj_buf[1]);
   acc_code = acc_mask = 0xffffffff;
   new_init = TRUE;

   nbr_of_obj = 0;
   for (i=0; i<=MAX_BIND_OBJ-1; i++) bind_index_list[i] = 0;
}



/*---------------------------------------------------------------------*
 *     C _ D E F I N E _ O B J
 *---------------------------------------------------------------------*
 *  Funktion:
 *     Definiert ein neues Objekt im Objekt-Buffer und berechnet die
 *         resultierende Akzeptanz-Maske (beim Philips SJA1000 Controller).
 *
 *---------------------------------------------------------------------*
 *  Parameter:
 *
 *     identifier   : Identifikation des zu definierenden Objektes
 *     data_length  : Anzahl Datenbytes des zu definierenden
 *                    Objektes (0..8)
 *     obj_typ      : Genauere Beschreibung der Verwendungsart des
 *                    Objektes :
 *
 *                    D_REC        :   Objekt kann nur empfangen werden
 *                    D_SEND       :   Objekt kann nur gesendet werden
 *                    D_REC_R_SEND :   Objekt kann empfangen werden
 *                                     und es kann ein entsprechende
 *                                     Remote-Frame ausgesendet werden
 *                    D_SEND_R_REC :   Objekt kann gesendet werden und es
 *                                     kann ein entsprechendes Remote-Frame
 *                                     empfangen und beantwortet werden
 *
 *
 *     Returnwert   : OK, OBJ_ERROR, TOO_LONG, INVALID_TYPE, MEM_FULL
 *
 *---------------------------------------------------------------------*/

byte c_define_obj (unsigned long identifier,
		   byte data_length,
		   byte obj_typ)
{
   /* Test der Input-Werte */
   if (data_length > MAX_OBJECT_DATA_SIZE) return C_TOO_LONG;
   if ((obj_typ != D_REC) && (obj_typ != D_SEND) &&
	 (obj_typ != D_REC_R_SEND) && (obj_typ != D_SEND_R_REC))
		 return C_INVALID_TYPE;
   if (nbr_of_obj >= obj_buf_len) return C_MEM_FULL;
   if (nbr_of_obj >= MAX_OBJECTS) return C_MEM_FULL;
   if (identifier > MAX_OBJECT_IDENT) return C_OBJ_ERROR;
   if (c_get_index_from_ident(identifier) == 0) {
      nbr_of_obj++;
      can_obj_buf[nbr_of_obj].identifier = identifier;
      can_obj_buf[nbr_of_obj].obj_typ = obj_typ;
      can_obj_buf[nbr_of_obj].data_length = data_length;
      can_obj_buf[nbr_of_obj].task_bind = NO_BIND;
      can_obj_buf[nbr_of_obj].obj_read = TRUE;
      /* Werte fr die Akkzeptanzfilterung berechnen */
      if (obj_typ != D_SEND) {
	 if (new_init) {
	    /* erstes Objekt */
	    acc_code = identifier;
	    new_init = FALSE;
	 }
	 else {
	    acc_mask = ~(acc_code ^ identifier) & acc_mask;
	    acc_code = acc_code & identifier;
	 }
      }
      return C_OK;
   }
   else {
      /* Objekt existiert schon */
      return C_OBJ_ERROR;
   }

}


#pragma eject

/*---------------------------------------------------------------------*
*         C _ G E T _ A C C _ C O D E
*---------------------------------------------------------------------*
*  Funktion:
*         Gibt den beim Definieren der Objekte (mit C_DEFINE_OBJ)
*         berechneten Acceptance-Code fr den SJA1000 zurck.
*
*---------------------------------------------------------------------*
*  Parameter:
*
*         Returnwert   : Acceptance-Code fr den SJA1000
*
*---------------------------------------------------------------------*/

byte c_get_acc_code (void)
{
   /* Nur die 8 MSB des acc_code zurckgeben */
   return (acc_code >> 3);
}



/*---------------------------------------------------------------------*
*         C _ G E T _ A C C _ M A S K
*---------------------------------------------------------------------*
*  Funktion:
*         Gibt die beim Definieren der Objekte (mit C_DEFINE_OBJ)
*         berechnete Acceptance-Maske fr den SJA1000 zurck.
*
*---------------------------------------------------------------------*
*  Parameter:
*
*         Returnwert   : Acceptance-Maske fr den SJA1000
*
*---------------------------------------------------------------------*/

byte c_get_acc_mask (void)
{
   /* Nur die 8 MSB der acc_mask zurckgeben */
   return ~(acc_mask >> 3);
}




#pragma eject
/*---------------------------------------------------------------------*
 *     Zugriffsfunktionen                                              *
 *---------------------------------------------------------------------*/

 /* Die Applikation benutzt die Objekt-Identifikation fr den Zugriff  */
 /* auf ein bestimmtes Objekt. Intern wird jedoch nur der Index eines  */
 /* bestimmten Objektes innerhalb des Objektbuffers zum Zugriff        */
 /* benutzt. Mit den Prozeduren C_GET_INDEX und C_GET_IDENT    */
 /* kann der Index resp. die Identifikation eines bestimmten Objektes  */
 /* ermittelt werden.                                                  */
 /*                                                                    */
 /* Die jeweiligen Zugriffsprozeduren testen aus Zeitgrnden jeweils   */
 /* nicht mehr ob ein bestimmter Index tatschlich einem definierten   */
 /* Objekt entspricht !                                                */


/*---------------------------------------------------------------------*
 *         C _ G E T _ I N D E X _ F R O M _ I D E N T
 *---------------------------------------------------------------------*
 *  Funktion:
 *     Testet ob das Objekt IDENTIFIER im Objekt-Buffer definiert ist
 *     und gibt den entsprechenden Index innerhalb CAN_OBJ_BUF zurck.
 *
 *---------------------------------------------------------------------*
 *  Parameter:
 *
 *     identifier   :    Identifikation des gesuchten Objektes
 *
 *     Returnwert   :    0        -->   Das Objekt IDENTIFIER existiert
 *                                      nicht
 *                       1 .. 255 -->   Index des Objektes IDENTIFIER
 *
 *---------------------------------------------------------------------*/

byte c_get_index_from_ident (unsigned long identifier)
{
   byte i, temp;

   temp = nbr_of_obj;
   for (i=1; i<=temp; i++) {
      if (can_obj_buf[i].identifier == identifier) return i;
   }
   return 0;
}



/*---------------------------------------------------------------------*
 *     C _ G E T _ I D E N T
 *---------------------------------------------------------------------*
 *  Funktion:
 *     Gibt den Identifier des Objektes mit dem INDEX innerhalb
 *     CAN_OBJ_BUF zurck.
 *
 *---------------------------------------------------------------------*
 *  Parameter:
 *
 *     index        :    Index des gesuchten Objektes
 *
 *     Returnwert   :    0 .. 7EFH  ->  Identifikation des gewnschten
 *                       Objektes
 *                       FFFFH      ->  Objekt existiert nicht
 *
 *---------------------------------------------------------------------*/

unsigned long c_get_ident (byte index)
{
   if ((index == 0) || (index > nbr_of_obj)) return 0xffff;
   return can_obj_buf[index].identifier;
}



/*---------------------------------------------------------------------*
 *     C _ G E T _ R E C _ B I N D
 *---------------------------------------------------------------------*
 *  Funktion:
 *     Gibt den Index eines neu empfangenen und noch nicht von der
 *     Applikation gelesenen Objektes, das fest an den Task TASK_ID
 *     gebunden ist, zurck.
 *
 *---------------------------------------------------------------------*
 *  Parameter:
 *
 *     task_id   :    Task-Identifikation des Tasks an den das Objekt
 *                    gebunden ist
 *
 *     Returnwert   :    0        -->   Es existiert kein gewnschtes Objekt
 *                       1 .. 255 -->   Index des Objektes
 *
 *---------------------------------------------------------------------*/

byte c_get_rec_bind (byte task_id)
{
   byte i, ind;

   for (i=0; i<=MAX_BIND_OBJ-1; i++) {
      ind = bind_index_list[i];
      if (ind != 0) {
	 if (can_obj_buf[ind].task_bind == task_id) {
	    if (!(can_obj_buf[ind].obj_read)) {
	       return ind;
	    }
	 }
      }
   }
   return 0;
}



/*---------------------------------------------------------------------*
 *     C _ G E T _ O B J _ T Y P
 *---------------------------------------------------------------------*
 *  Funktion:
 *     Gibt den Objekttyp wie in C_DEFINE_OBJ definiert des Objektes
 *     INDEX zurck.
 *
 *---------------------------------------------------------------------*
 *  Parameter:
 *
 *     index        :    Index des Objektes innerhalb CAN_OBJ_BUF dessen
 *                       Typ gelesen werden soll
 *
 *     Returnwert   :    Objekttyp des Objektes INDEX
 *
 *---------------------------------------------------------------------*/

byte c_get_obj_typ (byte index)
{
   return can_obj_buf[index].obj_typ;
}



/*---------------------------------------------------------------------*
 *     C _ W R I T E _ D A T A
 *---------------------------------------------------------------------*
 *  Funktion:
 *     Schreiben von Daten aus dem Array ARR in das Objekt mit INDEX
 *     im Objekt-Buffer.
 *     Es werden soviele Datenbytes geschrieben wie DATA_LENGTH
 *     des entsprechenden Objektes angibt.
 *         Die Daten werden nur in das Objekt geschrieben, sie werden
 *     nicht gesendet.
 *
 *---------------------------------------------------------------------*
 *  Parameter:
 *
 *     index      :      Index des Objektes innerhalb OBJ_BUF in das
 *                       geschrieben werden soll
 *
 *     arr        :      Zu schreibende Daten
 *
 *---------------------------------------------------------------------*/

void c_write_data (byte index,
		   byte xdata *arr)
{
   if (can_obj_buf[index].data_length > 0) {
      c_fast_copy (arr, &can_obj_buf[index].dat[0],
		   can_obj_buf[index].data_length);

   }
}



/*---------------------------------------------------------------------*
 *     C _ R E A D _ D A T A
 *---------------------------------------------------------------------*
 *  Funktion:
 *     Lesen der Daten des Objektes INDEX im Objekt-Buffer. Es werden
 *     soviele Daten gelesen, wie DATA_LENGTH des entsprechenden
 *     Objektes angibt.
 *
 *---------------------------------------------------------------------*
 *  Parameter:
 *
 *     index       :  Index des Objektes in OBJ_BUF aus dem gelesen
 *                    werden soll.
 *
 *     arr         :  Array in den die Daten gelesen werden sollen
 *
 *
 *---------------------------------------------------------------------*/

void c_read_data (byte index,
		  byte xdata *arr)
{
byte length;

   length = can_obj_buf[index].data_length;
   if (length > 0) {
      c_fast_copy (&can_obj_buf[index].dat[0], arr, length);
   }
   can_obj_buf[index].obj_read = TRUE;
}



/*---------------------------------------------------------------------*
 *     C _ R E C E I V E _ O B J
 *---------------------------------------------------------------------*
 *  Funktion:
 *     Fhrt die nach einem Receive-Interrupt ntigen Funktionen aus.
 *
 *---------------------------------------------------------------------*
 *  Parameter:
 *
 *         Returnwert beim Philips SJA1000 :
 *         ==============================
 *                                        Normale Rckmeldung bei einwandfrei Empfangenen
 *                    Data-Frames :
 *                    1 bis 255   ->    Index innerhalb OBJ_BUF des
 *                                      empfangenen und abgespeicherten
 *                                                                              Objektes
 *
 *                                                                              Rckmeldungen falls kein
 *                                                                              Data-Frame fr die Applikation
 *                                      empfangen :
 *
 *                                        FFF0H           ->    Ein Remote-Frame wurde empfangen und
 *                                      beantwortet
 *                    FFF1H       ->    Ein Remote-Frame wurde empfangen,
 *                                      das entsprechende Objekt hat aber
 *                                      einen falschen Typ
 *                    FFF2H       ->    Ein Remote-Frame wurde empfangen,
 *                                      die Antwort konnte aber wegen
 *                                      Bus-Problemen nicht gesendet werden
 *                    FFF3H       ->    Das Objekt hat die 2. Stufe der
 *                                      Akzeptanzfilterung nicht passiert
 *                    FFF4H       ->    Das Objekt hat die 3. Stufe der
 *                                      Akzeptanzfilterung nicht passiert
 *                    FFF5H       ->    Ein Objekt wurde empfangen und
 *                                      akzeptiert, das entsprechende
 *                                      Objekt hat aber einen falschen Typ
 *                    FFF6H       ->    Ein Objekt wurde empfangen,
 *                                      akzeptiert und abgespeichert,
 *                                      die Applikation muss aber nicht
 *                                      benachrichtigt werden, da das
 *                                      Objekt nur neu beschrieben wurde
 *                                      (seit dem letzten Empfang noch
 *                                      nicht gelesen durch die
 *                                      Applikation).
 *
 *
 *
 *
 *---------------------------------------------------------------------*/

unsigned int c_receive_obj (void)
{
unsigned long ident;
byte index;
byte res;

   ident = c_get_rec_id();
   /* 2. Stufe der Akzeptanzfilterung */
   if (!(c_obj_accept(ident))) {
      c_rel_rec_reg();
      return 0xfff3;
   }
   /* 3. Stufe der Akzeptanzfilterung */
   index = c_get_index_from_ident(ident);
   if (index == 0) {
      c_rel_rec_reg();
      return 0xfff4;
   }

   /* Das Objekt hat die Akzeptanzfilterung bestanden   */
   /* Test ob Remote-Frame                                                         */
   if (c_rec_remote()) {
      /* Test ob Antwort auf Remote-Frame gesendet werden darf */
      if (can_obj_buf[index].obj_typ == D_SEND_R_REC) {
	 /* Empfangsbuffer freigeben und Antwort senden */
	 c_rel_rec_reg();
	 res = c_send_obj (ident, FALSE, can_obj_buf[index].data_length,
			  can_obj_buf[index].dat);
	 if (res==TRUE) {
	    return 0xfff0;
	 }
	 else {
	    return 0xfff2;
	 }
      }
      else {
	 /* Falscher Objekttyp */
	 c_rel_rec_reg();
	 return 0xfff1;
      }
   }
   else {
      /* Data-Frame empfangen */
      if ((can_obj_buf[index].obj_typ == D_REC) ||
	 (can_obj_buf[index].obj_typ == D_REC_R_SEND)) {
	 /* Daten empfangen */
	 c_read_rec_data (can_obj_buf[index].dat);
	 c_rel_rec_reg();
	 if (can_obj_buf[index].obj_read) {
	    can_obj_buf[index].obj_read = FALSE;
	    return index;
	 }
	 else {
	    return 0xfff6;
	 }
      }
      else {
	 /* falscher Objekttyp */
	 c_rel_rec_reg();
	 return 0xfff5;
      }
   }
}



/*---------------------------------------------------------------------*
 *     C _ T R A N S M I T _ O B J
 *---------------------------------------------------------------------*
 *  Funktion:
 *         Senden eines Data-Frames ber den CAN-Bus mit Abspeichern der
 *         Daten im Objektbuffer.
 *
 *---------------------------------------------------------------------*
 *  Parameter:
 *
 *         Index           :  Index des zu sendenden Objektes
 *
 *     arr         :  Array mit den Daten die gesendet werden sollen
 *
 *     Returnwert  :  TRUE  --> Objekt erfolgreich gesendet
 *                    FALSE --> Objekt konnte nicht gesendet werden
 *                              (Bus-Fehler)
 *
 *---------------------------------------------------------------------*/

byte c_transmit_obj (byte index,
		     byte xdata *arr)
{
   c_write_data (index, arr);
   return c_send_obj (can_obj_buf[index].identifier, FALSE,
			 can_obj_buf[index].data_length,
			 arr);
}



/*---------------------------------------------------------------------*
 *     C _ S E N D _ R E M O T E
 *---------------------------------------------------------------------*
 *  Funktion:
 *     Senden eines eines Remote-Frames ber den CAN-Bus
 *
 *---------------------------------------------------------------------*
 *  Parameter:
 *
 *     Index       :  Index des Objektes von dem ein Remote-Frame
 *                    gesendet werden soll
 *
 *         Returnwert  :  TRUE  --> Objekt erfolgreich gesendet
 *                    FALSE --> Objekt konnte nicht gesendet werden
 *                              (Bus-Fehler)
 *
 *---------------------------------------------------------------------*/

byte c_send_remote (byte index)
{
   return c_send_obj (can_obj_buf[index].identifier, TRUE, 0, 0);
}



/*---------------------------------------------------------------------*
 *     C _ S E T _ B I N D
 *---------------------------------------------------------------------*
 *  Funktion:
 *     Setzt das Feld TASK_BIND innerhalb des Objektes INDEX im
 *     Objekt-Buffer auf eine bestimmte TASK_ID (wird zum festen
 *     Anbinden eines Objektes an einen Task bentigt, der CAN-Driver
 *     kann damit ermitteln, zu welchem Task er ein Signal senden muss,
 *     wenn dieses Objekt empfangen wird). Gleichzeitig wird der Index
 *     des Objektes im Array BIND_INDEX_LIST abgespeichert.
 *
 *---------------------------------------------------------------------*
 *  Parameter:
 *
 *     index      :  Index des Objekts bei dem der Task-Bind Wert
 *                   gesetzt werden soll
 *
 *     task_id    :  Wert auf den der Task-Bind Wert gesetzt werden soll
 *
 *     Returnwert  :  TRUE  -->   OK
 *                    FALSE -->   Die BIND_INDEX_LIST ist voll
 *                                (bereits 8 Objekte angebunden)
 *
 *---------------------------------------------------------------------*/

byte c_set_bind (byte index,
		 byte task_id)
{
   byte i;
   for (i=0; i<=MAX_BIND_OBJ-1; i++) {
      if ((bind_index_list[i] == 0) || (bind_index_list[i] == index)) {
	 bind_index_list[i] = index;
	 can_obj_buf[index].task_bind = task_id;
	 return TRUE;
      }
   }
   return FALSE;
}



/*---------------------------------------------------------------------*
 *     C _ C L E A R _B I N D
 *---------------------------------------------------------------------*
 *  Funktion:
 *     Markiert das Objekt als zu keinem bestimmten Task zugehrig.
 *     Setzt das Feld TASK_BIND innerhalb des Objektes INDEX im
 *     Objekt-Buffer auf FFH (in RTX-51 keine gltige TASK-ID).
 *
 *---------------------------------------------------------------------*
 *  Parameter:
 *
 *     index  : Index des Objekts bei dem der Task-Bind Wert gelscht
 *              werden soll
 *
 *---------------------------------------------------------------------*/

void c_clear_bind (byte index)
{
   byte i;

   for (i=0; i<=7; i++) {
      if (bind_index_list[i] == index) bind_index_list[i] = 0;
   }
   can_obj_buf[index].task_bind = NO_BIND;
}



/*---------------------------------------------------------------------*
 *     C _ G E T _ B I N D
 *---------------------------------------------------------------------*
 *  Funktion:
 *     Gibt den aktuellen Wert des Feldes TASK_BIND im Objekt
 *     INDEX zurck. NO_BIND bedeutet dabei, dass das Objekt an keinen
 *     Task gebunden ist.
 *
 *---------------------------------------------------------------------*
 *  Parameter:
 *
 *     index       :  Index des Objekts dessen Task-Bind Wert gelesen
 *                    werden soll
 *
 *     Returnwert  :  Alle Werte ausser FFH sind eine gltige Task-ID.
 *                    FFH bedeutet, dass das Objekt an keinen Task
 *                    gebunden ist.
 *
 *---------------------------------------------------------------------*/

byte c_get_bind (byte index)
{
   return can_obj_buf[index].task_bind;
}



/*---------------------------------------------------------------------*
 *     C _ S E T _ R E A D
 *---------------------------------------------------------------------*
 *  Funktion:
 *     Setzt OBJ_READ innerhalb des Objektes INDEX auf TRUE. Diese
 *     Funktion dient dazu ein neu empfangenes Objekt als von der
 *     Applikation gelesen zu markieren, falls die Applikation nicht
 *     getstartet werden kann (Mailbox voll usw.).
 *
 *---------------------------------------------------------------------*
 *  Parameter:
 *
 *     index       :  Index des Objekts bei dem OBJ_READ gesetzt
 *                    werden soll
 *
 *---------------------------------------------------------------------*/

void c_set_read (byte index)
{
   can_obj_buf[index].obj_read = TRUE;
}



/*---------------------------------------------------------------------*
 *     C _ S E T _ A L L _ R E A D
 *---------------------------------------------------------------------*
 *  Funktion:
 *     Setzt OBJ_READ innerhalb aller Objekte auf TRUE. Diese
 *     Funktion dient dazu alle empfangenen Objekte als von der
 *     Applikation gelesen zu markieren.
 *
 *---------------------------------------------------------------------*/

void c_set_all_read (void)
{
   byte i;

   for (i=1; i<=nbr_of_obj; i++) {
      can_obj_buf[i].obj_read = TRUE;
   }
}



#pragma eject
/*---------------------------------------------------------------------*
*     Akzeptanzfilterung                                              *
*---------------------------------------------------------------------*/

/*---------------------------------------------------------------------*
*         C _ O B J _ A C C E P T
*---------------------------------------------------------------------*
*  Funktion:
*         Fhrt eine volle 11-Bit Akezptanzfilterung durch, wie beim
*         Philips SJA1000 definiert (der SJA1000 vergleicht nur die
*         obersten 8-Bit). Die Filterwerte werden bei C_DEFINE_OBJ
*         berechnet.
*
*---------------------------------------------------------------------*
*  Parameter:
*
*         identifier   : Identifikation die getestet werden soll
*
*         Returnwert   : TRUE  ->        Filter durchlaufen (muss aber nicht
*                                                                bedeuten, dass das Objekt im Objektbuffer
*                                                                definiert ist, die Filterung ist nur eine
*                                                                Vorauswahl.
*
*                                        FALSE ->        Filterung nicht bestanden, Objekt ist
*                                                                nicht im Buffer definiert.
*
*---------------------------------------------------------------------*/

byte c_obj_accept (unsigned long identifier)
{
   if (((identifier ^ acc_code) & (acc_mask)) == 0) {
      return TRUE;
   }
   else
   {
      return FALSE;
   }
}


/*---------------------------------------------------------------------*
 *     Ende Modul BCANOBJA                                             *
 *---------------------------------------------------------------------*/
