#pragma large
#pragma debug
#pragma registerbank(3)
#pragma pagelength(80) pagewidth(110)
/***********************************************************************
************************************************************************
*                                                                      *
*  F U L L - C A N   C O N T R O L L E R                               *
*                                                                      *
************************************************************************
************************************************************************
*                                                                      *
*  Funktion:                                                           *
*                                                                      *
*     Direkte HW-Steuerung des Intel 82526 Full-CAN Controllers.       *
*     Ausfhrung aller Low-Level Funktionen. Dieses Modul ist das      *
*     einzige, dass direkt in die HW-Register des CAN-Controllers      *
*     schreiben darf.                                                  *
*                                                                      *
*     Die Adressierung der einzelnen Objekte im DPRAM des Controllers  *
*     erscheint im ersten Moment vielleicht ein wenig simpel           *
*     gelst. Da bei dieser Implementation jedoch vorallem bezglich   *
*     des Laufzeitverhaltens optimiert wurde, liess sich ein gewisses  *
*     Mass an fehlender Eleganz nicht vermeiden.
*                                                                      *
*                                                                      *
*  Filename           :  FCANCONT.C51                                  *
*                                                                      *
*  Modulename         :  FCANCONT                                      *
*                                                                      *
*  Zielsystem         :  jedes 8051 System mit einem Intel 82526       *
*                        CAN-Controller                                *
*                                                                      *
*  Entwicklungssystem :  Keil C-51                                     *
*                                                                      *
************************************************************************
*  Versionen:                                                          *
*                                                                      *
*  1.0  28.01.1991  Th. Fischler : Erste Version                       *
*                                                                      *
************************************************************************
*    Copyright 1991 .. 2001 METTLER  &  FUCHS  AG,  CH-8953 Dietikon   *
************************************************************************/

/*---------------------------------------------------------------------*
 *     D E F I N I T I O N E N                                         *
 *---------------------------------------------------------------------*/

#include "abbrev.h"
#include "fcancont.h"
#include "candefs.h"

/* Maximale Anzahl der Sende-Versuche bei Bus-Problemen */
/* von BCACNCONF.A51                                    */
extern const unsigned int code NBR_OF_SEND_TRIALS;


/* Definition der einzelnen Bits im STATUS-REGISTER    */
/* des CAN-Controllers                                 */
union can_status_typ {
   struct {
      byte reset_request    : 1;
      byte halt_status      : 1;
      byte error_status     : 1;
      byte transmit_status  : 1;
      byte rec_status       : 1;
      byte bus_status       : 1;
      byte dpram_status     : 1;
      byte                  : 1;
   } status_bits;
   byte status_byte;
};



/*---------------------------------------------------------------------*
 *     I M P O R T S                                                   *
 *---------------------------------------------------------------------*/

#include "canutil.h"


/*---------------------------------------------------------------------*
 *     M O D U L G L O B A L E  V A R I A B L E N                      *
 *---------------------------------------------------------------------*/

   /* Definition der Intel 82526 HW-Register, diese werden an die    */
   /* Speicherstelle des Symbols FCAN_REG im Assembler-Modul         */
   /* FCANCONF.A51 alloziert. (absolute Variable)                    */
   extern xdata volatile union {
                           struct {
                              byte control;
                              byte status;
                              byte intr;
                              byte bus_tim_0;
                              byte bus_tim_1;
                              byte out_contr;
                           } r;           /* Allgemeine Register im DPRAM */
                           byte mem[63];  /* Ganzes DPRAM als Array       */
                        } fcan_reg;



#pragma eject
/*---------------------------------------------------------------------*
 *     L O K A L E  P R O Z E D U R E N                                *
 *---------------------------------------------------------------------*/

/*---------------------------------------------------------------------*
 *     C A N _ R E S E T _ R E Q U E S T
 *---------------------------------------------------------------------*
 *  Funktion:
 *     Setzt den CAN-Controller in den Reset-Zustand
 *
 *---------------------------------------------------------------------*
 *  Parameter:
 *     --
 *
 *---------------------------------------------------------------------*/

static void can_reset_request (void)
{
   fcan_reg.r.control = fcan_reg.r.control | 0x01;
   /* Testen ob ein Reset-Request whrend dem Betrieb verlangt wurde */
   /* und solange warten, bis Reset-Zustand quittiert wird           */
   while (!(fcan_reg.r.status & 0x01)) {
      /* Falls ntig den Interrupt-Pointer zurcksetzen */
      if (fcan_reg.r.intr != 0xff) {
         fcan_reg.r.intr = 0xff;
      }
   }
   /* Falls ntig den Interrupt-Pointer zurcksetzen */
   if (fcan_reg.r.intr != 0xff) {
      fcan_reg.r.intr = 0xff;
   }
}



/*---------------------------------------------------------------------*
 *     C A N _ H A L T _ R E Q U E S T
 *---------------------------------------------------------------------*
 *  Funktion:
 *     Setzt den CAN-Controller in den Halt-Zustand
 *
 *---------------------------------------------------------------------*
 *  Parameter:
 *     --
 *
 *---------------------------------------------------------------------*/

static void can_halt_request (void)
{
   /* Status Register testen ob Controller Bus-Off ist */
   if (fcan_reg.r.status & 0x20) {
      /* Controller ist Bus-Off, Halt alleine gengt nicht */
      /* Es muss ein Reset ausgelst werden                */
      can_reset_request();
   }
   /* Nur Halt-setzen falls nicht schon reset gesetzt ist */
   if (!(fcan_reg.r.status & 0x01)) {
      fcan_reg.r.control = fcan_reg.r.control | 0x02;
      /* Nun Abwarten bis der Haltzustand vom Controller quittiert wird */
      while (!(fcan_reg.r.status & 0x02));
   }
}



/*---------------------------------------------------------------------*
 *     C A N _ S E T _ I M P
 *---------------------------------------------------------------------*
 *  Funktion:
 *     Setzt das IMP-Access-Flag des durch OBJ_ADR adressierten Objektes
 *
 *---------------------------------------------------------------------*
 *  Parameter:
 *
 *     obj_adr        :  Interne Start-Adresse des Objektes
 *                       (siehe C_INIT_OBJ)
 *
 *---------------------------------------------------------------------*/

static void can_set_imp (byte obj_adr)
{
   fcan_reg.mem[obj_adr] = fcan_reg.mem[obj_adr] | 0x80;
}



/*---------------------------------------------------------------------*
 *     C A N _ C L E A R _ I M P
 *---------------------------------------------------------------------*
 *  Funktion:
 *     Lscht das IMP-Access-Flag des durch OBJ_ADR adressierten Objektes
 *
 *---------------------------------------------------------------------*
 *  Parameter:
 *
 *     obj_adr        :  Interne Start-Adresse des Objektes
 *                       (siehe C_INIT_OBJ)
 *
 *---------------------------------------------------------------------*/

static void can_clear_imp (byte obj_adr)
{
   fcan_reg.mem[obj_adr] = fcan_reg.mem[obj_adr] & 0x7f;
}




/*---------------------------------------------------------------------*
 *     C A N _ C H E C K _ W R I T E _ A C C E S S
 *---------------------------------------------------------------------*
 *  Funktion:
 *     Prft ob auf ein Objekt schreibend zugegriffen werden darf
 *     Falls zugegriffen werden darf, ist der IMP-Access nach dieser
 *     Routine gesperrt.
 *
 *---------------------------------------------------------------------*
 *  Parameter:
 *
 *     obj_adr        :  Interne Start-Adresse des Objektes
 *                       (siehe C_INIT_OBJ)
 *
 *     Returnwert     :  TRUE  --> Zugriff ok
 *                       FALSE --> Zugriff verweigert
 *
 *---------------------------------------------------------------------*/

static byte can_check_write_access (byte obj_adr)
{
   unsigned int nbr_of_trials;  /* Anzahl Sendeversuche bis Abbruch */

   nbr_of_trials = 0;
   can_clear_imp (obj_adr);
   if (fcan_reg.mem[obj_adr+2] & 0x80) {
      /* TX-Request = 1, das Objekt ist noch am Senden */
      /* warten bis fertig                             */
      can_set_imp (obj_adr);
      while (fcan_reg.mem[obj_adr+2] & 0x80) {
         /* while TX-Request=1 */
         nbr_of_trials++;
         if (nbr_of_trials > NBR_OF_SEND_TRIALS) {
            /* Vermutlich kein anderer Controller am Bus */
            /* Abbrechen                                 */
            return FALSE;
         }
      }
      nbr_of_trials = 0;
      can_clear_imp (obj_adr);
   }  /* end if */
   /* Das Objekt wre bereit zum Senden, prfen ob CPU-Access */
   if (!(fcan_reg.mem[obj_adr+1] & 0x01)) {
      /* CPU-Access ist nicht frei */
      if (fcan_reg.mem[obj_adr+1] & 0x02) {
         /* Remote-Frame ist gerade eingetroffen, Controller */
         /* will senden                                      */
         can_set_imp (obj_adr);
      }
      /* Nun warten bis CPU-Access frei ist */
      while (!(fcan_reg.mem[obj_adr+1] & 0x01)) {
         nbr_of_trials++;
         if (nbr_of_trials > NBR_OF_SEND_TRIALS) {
            /* Vermutlich kein anderer Controller am Bus */
            /* Abbrechen                                 */
            return FALSE;
         }
      }
      can_clear_imp (obj_adr);
   }  /* end if */
   return TRUE;
}



#pragma eject
/*---------------------------------------------------------------------*
 *     E X P O R T I E R T E  P R O Z E D U R E N                      *
 *---------------------------------------------------------------------*/

/*---------------------------------------------------------------------*
 *     C _ F I R S T _ I N I T
 *---------------------------------------------------------------------*
 *  Funktion:
 *     Beim Starten des 82526 nach Power-Down ergeben sich manchmal
 *     kleinere Probleme den Chip sauber zu initialisieren.
 *     Diese Routine lst dieses Problem, indem der 82526 zweimal
 *     geresetet wird. Diese Routine sollte vor der ersten
 *     initialisierung des Controllers aufgerufen werden.
 *
 *---------------------------------------------------------------------*
 *  Parameter:
 *     --
 *---------------------------------------------------------------------*/
void c_first_init (void)
{
   fcan_reg.r.control = 0x01;  /* Reset */
   fcan_reg.r.control = 0x00;  /* Reset aufheben */
   fcan_reg.r.control = 0x01;  /* Reset */
   /* Interrupt-Pointer zurcksetzen */
   fcan_reg.r.intr = 0xff;
}



/*---------------------------------------------------------------------*
 *     C _ H W _ I N I T
 *---------------------------------------------------------------------*
 *  Funktion:
 *     Grundinitialisierung des 82526 CAN-Controllers.
 *     Der CAN-Controller wird fr diese Operation angehalten.
 *     Er muss danach mit C_START wieder neu gestartet werden.
 *
 *---------------------------------------------------------------------*
 *  Parameter:
 *
 *     bus_t0_reg   : Bus-Timing Register 0 des 82526
 *     bus_t1_reg   : Bus-Timing Register 1 des 82526
 *     out_c_reg    : Output-Control Register des 82526
 *
 *     Returnwert   : TRUE   -> OK
 *                    FALSE  -> Fehler beim Init
 *---------------------------------------------------------------------*/

byte c_hw_init (byte bus_t0_reg,
                byte bus_t1_reg,
                byte out_c_reg)
{
   /* Plausibilittskontrolle der Parameter noch weggelassen */
   can_reset_request();
   /* Operation-Bit auf low setzen */
   fcan_reg.r.control = (fcan_reg.r.control & 0x7f);
   fcan_reg.r.bus_tim_0 = bus_t0_reg;
   fcan_reg.r.bus_tim_1 = bus_t1_reg;
   fcan_reg.r.out_contr = out_c_reg;
   fcan_reg.r.intr = 0xff;
   return TRUE;
}



/*---------------------------------------------------------------------*
 *     C _ I N I T _ C O N T _ R E G
 *---------------------------------------------------------------------*
 *  Funktion:
 *     Initialisierung des CONTROL-REGISTERS des 82526
 *     (Festlegung der Interrupts und des Synch-Modes).
 *     Der CAN-Controller wird fr diese Operation angehalten, er muss
 *     danach mit C_START wieder neu gestartet werden.
 *
 *---------------------------------------------------------------------*
 *  Parameter:
 *
 *     cont_reg    :  Inhalt des 82526 Control-Registers
 *
 *---------------------------------------------------------------------*/

void c_init_cont_reg (byte cont_reg)
{
   /* Reset-Request einschalten */
   can_reset_request();
   /* Sicherstellen dass Halt-/Reset-Request und Operation nicht  */
   /* gendert wird                                               */
   fcan_reg.r.control = (cont_reg | 0x03) & 0x7F;
}



/*---------------------------------------------------------------------*
 *     C _ I N I T _ O B J
 *---------------------------------------------------------------------*
 *  Funktion:
 *     Definiert an der internen CAN-Controller Adresse BUF_ADR ein
 *     neues CAN-Objekt. Der CAN-Controller wird fr diese Operation
 *     angehalten, er muss danach mit C_START wieder neu gestartet
 *     werden.
 *
 *---------------------------------------------------------------------*
 *  Parameter:
 *
 *     ident       :  Identifier des zu definierenden Objektes
 *                    (0 .. 7EFH)
 *     data_len    :  Anzahl Datenbytes des Objektes (0 .. 8)
 *     obj_typ     :  Genaue Beschreibung der Objektart :
 *                    D_REC (0)         :  Objekt kann nur empfangen werden
 *                    D_SEND (1)        :  Objekt kann nur gesendet werden
 *                    D_REC_R_SEND (2)  :  Objekt kann empfangen werden
 *                                         und es kann ein entsprechendes
 *                                         Remote-Frame ausgesendet werden
 *                    D_SEND_R_REC (3)  :  Objekt kann gesendet werden und
 *                                         es kann ein entsprechendes
 *                                         Remote-Frame empfangen und
 *                                         beantwortet werden
 *     obj_adr     :  Interne Start-Adresse des Objektes im CAN-Controller,
 *                    das erste Byte des CAN-Controllers ist
 *                    dabei 0 (=Control Register), das erste Objekt beginnt
 *                    beim 82526 bei der Adresse 6H.
 *                    (Diese Art der Objekt-Adressierung drngt sich durch
 *                    die interne Struktur des 82526 Controllers auf,
 *                    so wird zum Beispiel bei einem Empfangsinterrupt
 *                    vom 82526 die Objektadresse des empfangenen Objektes
 *                    in einem Register zur Verfgung gestellt.
 *                    Das bergeordnete Modul muss bei dieser Art der
 *                    Objektverwaltung eine Liste der einzelnen Objekte
 *                    und ihre Zuordnung zu den internen Adressen fhren.)
 *
 *---------------------------------------------------------------------*/

void c_init_obj (unsigned int ident,
                 byte         data_len,
                 byte         obj_typ,
                 byte         obj_adr)
{
   /* Definition der einzelnen Bits in den Objekt-Descriptoren.    */
   /* Es werden nur die Bits definiert die sinnvollerweise einzeln */
   /* angesprochen werden.                                         */
   /* Im nachfolgenden Code wird auf die hier nicht definierten    */
   /* Bits direkt zugegriffen                                      */
   /* Diese Definition erleichtert die korrekte Initialisierung    */
   /* der Descriptoren                                             */
   union {
      struct {
         byte cpu_access        : 1;
         byte remote_tx_req     : 1;
         byte data_direction    : 1;
         byte remote_direction  : 1;
         byte                   : 4;
      } d2_bits;
      byte d2_byte;
   } desc2;

   union {
      struct {
         byte                   : 5;
         byte transfer_int_ena  : 1;
         byte transfer_status   : 1;
         byte transmission_req  : 1;
      } d3_bits;
      byte d3_byte;
   } desc3;

   /* Hilfsvariable zum separaten ansprechen von High- und Low-Byte  */
   /* in einem Word                                                  */
   register union {
               unsigned int tw;
               byte tb[2];
            } temp;

   /* Halt-Request aktivieren */
   can_halt_request();

   /* Descriptor Byte 1 des Objektes initialisieren */
   temp.tw = ident << 4;
   /* obere 7-Bit des Identifiers, IMP-Access freigeben */
   fcan_reg.mem[obj_adr] = (temp.tb[0]);
   can_set_imp (obj_adr);

   /* Descriptor Byte 2 */
   desc2.d2_bits.cpu_access = 1;
   desc2.d2_bits.remote_tx_req = 0;
   /* Remote- und Data-Direction anhand des Objekt-Typs setzen */
   if (obj_typ == D_REC) {
      desc2.d2_bits.data_direction = 0;
      desc2.d2_bits.remote_direction = 1;
   }
   else if (obj_typ == D_SEND) {
      desc2.d2_bits.data_direction = 1;
      desc2.d2_bits.remote_direction = 0;
   }
   else if (obj_typ == D_REC_R_SEND) {
      desc2.d2_bits.data_direction = 0;
      desc2.d2_bits.remote_direction = 1;
   }
   else if (obj_typ == D_SEND_R_REC) {
      desc2.d2_bits.data_direction = 1;
      desc2.d2_bits.remote_direction = 0;
   }
   /* Descriptor 2 schreiben */
   fcan_reg.mem[obj_adr+1] = ((desc2.d2_byte & 0x0f) | (temp.tb[1] & 0xf0));

   /* Descriptor Byte 3 */
   /* Wenn Empfangsobjekt, Interrupt freigeben */
   if ((obj_typ == D_REC) || (obj_typ == D_REC_R_SEND)) {
	  desc3.d3_bits.transfer_int_ena = 1;
   }
   else {
	  desc3.d3_bits.transfer_int_ena = 0;
   }
   /* Transfer-Status und Transmission Request auf Null setzen */
   desc3.d3_bits.transfer_status = 0;
   desc3.d3_bits.transmission_req = 0;
   /* Descriptor 3 schreiben */
   fcan_reg.mem[obj_adr+2] = ((desc3.d3_byte & 0xe0) | (data_len & 0x0f));
}



/*---------------------------------------------------------------------*
 *     C _ S T A R T
 *---------------------------------------------------------------------*
 *  Funktion:
 *     Starten des CAN-Controllers nach Init oder nach C_STOP.
 *     Nach C_START knnen Objekte empfangen und gesendet werden.
 *     Falls ntig wird der Controller bei C_START nochmals kurz
 *     zurckgesetzt (reset) um einen eventuellen Bus-Off Zustand
 *     zu lschen.
 *
 *---------------------------------------------------------------------*
 *  Parameter:
 *
 *     --
 *
 *---------------------------------------------------------------------*/

void c_start (void)
{
   /* Status Register testen ob Controller Bus-Off ist */
   if (fcan_reg.r.status & 0x20) {
      /* Controller ist Bus-Off, nochmals reseten */
      can_reset_request();
   }
   /* Eventuellen Halt-und Reset Zustand lschen */
   fcan_reg.r.control = fcan_reg.r.control & 0xfc;
   /* Nach einem Bus-Off bleibt der Reset fr 128*11 Bits erhalten */
   /* Warten bis Reset wirklich aufgehoben                         */
   while (fcan_reg.r.status & 0x01);
}



/*---------------------------------------------------------------------*
 *     C _ S T O P
 *---------------------------------------------------------------------*
 *  Funktion:
 *     Temporres Anhalten des CAN-Controllers. Es kann mit C_START
 *     weitergefahren werden. Solange C_STOP aktiv ist, knnen keine
 *     Objekte empfangen oder gesendet werden.
 *
 *---------------------------------------------------------------------*
 *  Parameter:
 *
 *     --
 *
 *---------------------------------------------------------------------*/

void c_stop (void)
{
   can_halt_request();
}



/*---------------------------------------------------------------------*
 *     C _ R E A D _ O B J
 *---------------------------------------------------------------------*
 *  Funktion:
 *     Lesen der Daten eines Objektes. Die Daten werden
 *     in den XDATA-Bereich, dessen Anfang mit dem Parameter BUF_PTR
 *     bezeichnet ist, kopiert. Es werden soviele Bytes kopiert wie bei
 *     C_INIT_OBJ definiert.
 *
 *---------------------------------------------------------------------*
 *  Parameter:
 *
 *     obj_adr     : Interne Start-Adresse des zu lesenden Objektes
 *                   (siehe C_INIT_OBJ)
 *
 *     arr         : Array im XDATA-Bereiche in den die
 *                   Daten kopiert werden knnen.
 *
 *---------------------------------------------------------------------*/

void c_read_obj (byte obj_adr,
                 byte xdata *arr)
{
   register byte len;

   /* Warten bis Zugriff frei ist (CPU-Access) */
   while (!(fcan_reg.mem[obj_adr+1] & 0x01));
   /* Daten-Lnge testen und entsprechend handeln */
   len = fcan_reg.mem[obj_adr+2] & 0x0f;
   if (len > 1) {
      /* Aufwendigeres Verfahren wenn Datenlnge > 1 Byte     */
      /* Es muss nach dem Lesen getestet werden ob in der     */
      /* Zwischenzeit weitere Daten empfangen wurden          */
      do {
         /* Transfer-Status = 0 */
         fcan_reg.mem[obj_adr+2] = fcan_reg.mem[obj_adr+2] & 0xbf;
         c_fast_copy (&fcan_reg.mem[obj_adr+3], arr, len);
         /* Nun testen ob keine neuen Daten eingetroffen sind */
         /* Warten bis Zugriff frei ist (CPU-Access) */
         while (!(fcan_reg.mem[obj_adr+1] & 0x01));
      } while (fcan_reg.mem[obj_adr+2] & 0x40);
   }
   else {
      /* Datenlnge ist <= 1, weiterer Empfang muss nicht getestet werden */
      /* Transfer-Status = 0 */
      fcan_reg.mem[obj_adr+2] = fcan_reg.mem[obj_adr+2] & 0xbf;
      c_fast_copy (&fcan_reg.mem[obj_adr+3], arr, len);
   }
}




/*---------------------------------------------------------------------*
 *     C _ S E N D _ O B J
 *---------------------------------------------------------------------*
 *  Funktion:
 *     Uebertragung eines Objektes zum CAN-Controller zum Senden.
 *     Es werden soviele Byte gesendet, wie bei C_INIT_OBJ definiert
 *     (nur wenn nicht Remote).
 *
 *---------------------------------------------------------------------*
 *  Parameter:
 *
 *     obj_adr        :  Interne Start-Adresse des zu sendenden Objektes
 *                       (siehe C_INIT_OBJ)
 *     remote         :  TRUE  -> sende ein Remote-Frame
 *                       FALSE -> sende ein Data-Frame
 *     arr            :  Array im XDATA-Bereich mit den Daten die gesendet
 *                       werden sollen
 *
 *     Returnwert     :  TRUE  -> Senden erfolgreich
 *                       FALSE -> Senden nicht erfolgreich
 *
 *---------------------------------------------------------------------*/

byte c_send_obj (byte obj_adr,
                 byte remote,
                 byte xdata *arr)
{
   if (remote) {
      /* Nur Remote Frame senden */
      if (fcan_reg.mem[obj_adr+1] & 0x01) {
         /* Es muss nur gesendet werden, falls CPU-Access=1 ist */
         /* Andernfalls wird gearade ein Data-Frame empfangen   */
         /* Es muss daher kein Remote-Frame gesendet werden     */
         /* Transfer-Status = 0 */
         fcan_reg.mem[obj_adr+2] = fcan_reg.mem[obj_adr+2] & 0xbf;
         /* TX-Request = 1 */
         fcan_reg.mem[obj_adr+2] = fcan_reg.mem[obj_adr+2] | 0x80;
      }
      return TRUE;
   }

   /* Daten-Frame senden */
   /* Testen ob Zugriff zum Schreiben auf Objekt */
   if (can_check_write_access (obj_adr)) {
      c_fast_copy (arr, &fcan_reg.mem[obj_adr+3],
                   fcan_reg.mem[obj_adr+2] & 0x0f);
      /* Transfer-Status = 0 */
      fcan_reg.mem[obj_adr+2] = fcan_reg.mem[obj_adr+2] & 0xbf;
      /* TX-Request = 1 */
      fcan_reg.mem[obj_adr+2] = fcan_reg.mem[obj_adr+2] | 0x80;
      can_set_imp (obj_adr);
      return TRUE;
   }
   return FALSE;
}



/*---------------------------------------------------------------------*
 *     C _ W R I T E _ O B J
 *---------------------------------------------------------------------*
 *  Funktion:
 *     Speichern eines Objektes im CAN-Controller, die Daten werden
 *     nicht gesendet sondern stehen zum Beispiel fr eine Anfrage
 *     durch ein Remote-Frame zur Verfgung. Es werden soviele Byte
 *     abgespeichert, wie bei C_INIT_OBJ definiert.
 *
 *---------------------------------------------------------------------*
 *  Parameter:
 *
 *     obj_adr        :  Interne Start-Adresse des zu schreibenden Objektes
 *                       (siehe C_INIT_OBJ)
 *     arr            :  Array im XDATA-Bereich mit den Daten die
 *                       geschrieben werden sollen
 *
 *     Returnwert     :  TRUE  -> Senden erfolgreich
 *                       FALSE -> Senden nicht erfolgreich
 *
 *---------------------------------------------------------------------*/

byte c_write_obj (byte obj_adr,
                  byte xdata *arr)
{
   if (can_check_write_access (obj_adr)) {
      c_fast_copy (arr, &fcan_reg.mem[obj_adr+3],
                   fcan_reg.mem[obj_adr+2] & 0x0f);
      /* Transfer-Status = 0 */
      fcan_reg.mem[obj_adr+2] = fcan_reg.mem[obj_adr+2] & 0xbf;
      can_set_imp (obj_adr);
      return TRUE;
   }
   return FALSE;
}



/*---------------------------------------------------------------------*
 *     C _ W R I T E _ E N D _ M A R K
 *---------------------------------------------------------------------*
 *  Funktion:
 *     Schreibt die Endmarke (FFH) an das Ende des Objektes OBJ_ADR.
 *     Diese Funktion dient zum Abschliessen der Objektliste.
 *
 *---------------------------------------------------------------------*
 *  Parameter:
 *
 *     OBJ_ADR        :  Interne Adresse des Objektes an dessen Ende die
 *                       Endmarke gesetzt werden soll(siehe C_INIT_OBJ)
 *
 *---------------------------------------------------------------------*/

void c_write_end_mark (byte obj_adr)
{
   byte len;

   can_halt_request();
   /* Lnge des Objektes berechnen */
   len = fcan_reg.mem[obj_adr+2] & 0x0f;
   fcan_reg.mem[obj_adr+3+len] = 0xff;
}



/*---------------------------------------------------------------------*
 *     C _ G E T _ I N T _ T Y P
 *---------------------------------------------------------------------*
 *  Funktion:
 *     Diese Funktion gibt den Typ des Aufgetretenen Interrupts zurck.
 *
 *---------------------------------------------------------------------*
 *  Parameter:
 *
 *     Returnwert  :  no_int, error_int, obj_int (enum interrupt_t)
 *
 *---------------------------------------------------------------------*/

enum interrupt_t c_get_int_typ (void)
{
   if (fcan_reg.r.intr == 0xff) {
      return no_int;
   }
   else if (fcan_reg.r.intr == 0x01) {
      return error_int;
   }
   else {
      return obj_int;
   }
}



/*---------------------------------------------------------------------*
 *     C _ G E T _ O B J _ I N T _ A D R
 *---------------------------------------------------------------------*
 *  Funktion:
 *     Gibt die Startadresse des Objektes bei dem ein Objekt-Interrupt
 *     aufgetreten ist zurck.
 *
 *---------------------------------------------------------------------*
 *  Parameter:
 *
 *     Returnwert  :  Startadresse des Objektes bei dem ein Interrupt
 *                    aufgetreten ist
 *
 *---------------------------------------------------------------------*/

byte c_get_obj_int_adr (void)
{
   return fcan_reg.r.intr;
}



/*---------------------------------------------------------------------*
 *     C _ R E C E I P T _ O B J _ I N T
 *---------------------------------------------------------------------*
 *  Funktion:
 *     Diese Funktion setzt einen Objekt-Interrupt des Objektes mit der
 *     internen Startadresse OBJ_ADR zurck.
 *
 *---------------------------------------------------------------------*
 *  Parameter:
 *
 *     OBJ_ADR        :  Interne Adresse des Objektes dessen Interrupt
 *                       zurckgesetzt werden soll (siehe C_INIT_OBJ)
 *
 *---------------------------------------------------------------------*/

void c_receipt_obj_int (byte obj_adr)
{
   /* Transfer-Status des Objektes = 0 */
   fcan_reg.mem[obj_adr+2] = fcan_reg.mem[obj_adr+2] & 0xbf;
   /* Empty-Pointer in das Interrupt Register schreiben */
   fcan_reg.r.intr = 0xff;
}


/*---------------------------------------------------------------------*
 *     C _ R E C E I P T _ E R R O R _ I N T
 *---------------------------------------------------------------------*
 *  Funktion:
 *     Diese Funktion quittiert einen Error-Interrupt
 *
 *---------------------------------------------------------------------*
 *  Parameter:
 *    --
 *---------------------------------------------------------------------*/

void c_receipt_error_int (void)
{
   /* Empty-Pointer in das Interrupt Register schreiben */
   fcan_reg.r.intr = 0xff;
}



/*---------------------------------------------------------------------*
 *     C _ G E T _ S T A T U S
 *---------------------------------------------------------------------*
 *  Funktion:
 *     Gibt den aktuellen Bus-Zustand des 82526.
 *
 *---------------------------------------------------------------------*
 *  Parameter:
 *
 *     Returnwert  :  Bus-Zustand des 82526
 *
 *---------------------------------------------------------------------*/

enum can_bus_stat_t c_get_status (void)
{
   union can_status_typ stat_r;   /* Status-Bits im CAN-Controller    */

   stat_r.status_byte = fcan_reg.r.status;
   if ((stat_r.status_bits.bus_status) ||
       (stat_r.status_bits.dpram_status)) {
      /* Der CAN-Controller ist auch bei einem DPRAM.Error */
      /* im Bus-Off Zustand                                */
      return bus_off;
   }
   if (stat_r.status_bits.error_status) {
      return error_passive;
   }
   return error_active;
}
